"""

Test for measure objects

Meant to be subclassed

"""
from django.contrib.auth.models import User
from django.test import TestCase

from installations import models as installations_models

from utils import tests as utils_tests
from utils.versioning.models import WorkVersion, Version

class MeasureTest(TestCase):
    fixtures = ['dummyusers', 'installations']
##    sc_model = Scenario
##    pl_model = ProtectionLayer
##    me_model = Measure
##    ap_model = AttentionPoint
##    sl_model = Suggestionlist
##    namespace = "degradations"

    def setUp(self):
        self.setPermissions()
        defaultuser = User.objects.get(pk=1)
        self.scope = installations_models.Scope.objects.filter(version__isnull=True)[0]
        self.scenario = self.sc_model(name='scenario', scope=self.scope)
        self.scenario.save(defaultuser)
        self.measure = self.me_model(name='Interlock', type='Safety Instrumented System')
        self.measure.save(defaultuser)
        self.attentionpoint = self.ap_model(name='Capacity', type='Specification', measure=self.measure)
        self.attentionpoint.save(defaultuser)
        self.protectionlayer = self.pl_model(name='PL')
        self.protectionlayer.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.scenario})
        self.plme = self.plme_model(protectionlayer=self.protectionlayer, measure = self.measure)
        self.plme.save(defaultuser)
        self.measure2 = self.me_model(name='PL2', type='PL')
        self.measure2.save(defaultuser)

        self.suggestionlist = self.sl_model(name='SL')
        self.suggestionlist.save(defaultuser)
        self.slscenario = self.sc_model(name='sl scenario', suggestionlist=self.suggestionlist)
        self.slscenario.save(defaultuser)
        self.slmeasure = self.me_model(name='PL', type='PL', SL=True)
        self.slmeasure.save(defaultuser)
        self.slattentionpoint = self.ap_model(name='Capacity', type='Specification', measure=self.slmeasure)
        self.slattentionpoint.save(defaultuser)

    def test_model(self):
        pass

    def test_model_delete(self):
        """
        test delete measure deletes attention points
        """
        me_c = self.me_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)
        measure = self.me_model(name='PL', type='PL')
        measure.save(defaultuser)
        attentionpoint = self.ap_model(name='Capacity', type='Specification', measure=measure)
        attentionpoint.save(defaultuser)

        self.assertEquals(self.me_model.objects.count(), me_c+1)
        self.assertEquals(self.ap_model.objects.count(), ap_c+1)

        self.me_model.objects.get(pk=measure.pk).delete(defaultuser)

        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

    def test_model_copy(self):
        me_c = self.me_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)
        measure = self.me_model(name='PL', type='PL')
        measure.save(defaultuser)
        attentionpoint = self.ap_model(name='Capacity', type='Specification', measure=measure)
        attentionpoint.save(defaultuser)

        self.assertEquals(self.me_model.objects.count(), me_c+1)
        self.assertEquals(self.ap_model.objects.count(), ap_c+1)

        # copy measure
        measure2 = measure.copy(defaultuser)

        self.assertEquals(self.me_model.objects.count(), me_c+2)
        self.assertEquals(self.ap_model.objects.count(), ap_c+2)

        self.assertNotEquals(measure2.v_id, measure.v_id)
        self.assertEquals(measure2.name, measure.name)

        # clean up
        self.me_model.objects.get(pk=measure.pk).delete(defaultuser)
        self.me_model.objects.get(pk=measure2.pk).delete(defaultuser)

        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

    def test_measure_views(self):

        base_url = '/analyses/%s/measures/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class measures(utils_tests.TestDefinition):
            description = 'list of protection layers'
            url = base_url
            users = [1,2,]
            contains = [{'url': base_url + 'add/', 'users': []},]

        class measures_report_select(utils_tests.TestDefinition):
            description = 'report of list of measures: select options'
            url = base_url + '?submit_pdf=submit_pdf&filter-name=x'
            users = [1,2,]

        class measures_report(utils_tests.postonly_TestDefinition):
            description = 'pdf of list of measures'
            url = base_url
            users = [1,2,]
            data = {
                "filter": "submit_pdf=submit_pdf&filter-name=x",
                "submit_pdf": "Generate pdf",
            }

        class measures_csv(utils_tests.postonly_TestDefinition):
            description = 'csv of list of measures'
            url = base_url
            users = [1,2,]
            data = {
                "filter": "filter-name=x",
                "submit_csv": "Generate csv",
            }

        class measure1_description(utils_tests.TestDefinition):
            description = 'display measure 1 tab description'
            url = base_url + '%s/description/' % self.measure.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.measure.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.measure.v_id, 'users': [1]},
            ]

        class measure2_description(utils_tests.TestDefinition):
            description = 'display measure 2 tab description'
            url = base_url + '%s/description/' % self.measure2.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.measure2.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.measure2.v_id, 'users': [1,]},
            ]

        class measure_attentionpoints(utils_tests.TestDefinition):
            description = 'display measure tab attention points'
            url = base_url + '%s/attentionpoints/' % self.measure.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/attentionpoints/add/' % self.measure.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.measure.v_id, 'users': []},
            ]

        class measure_usage(utils_tests.TestDefinition):
            description = 'display measure tab usage'
            url = base_url + '%s/usage/' % self.measure.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/delete/' % self.measure.v_id, 'users': []},
            ]

        class measure_compare(utils_tests.TestDefinition):
            description = 'compare measure'
            url = base_url + '%s/compare/0/' % self.measure.v_id
            users = [1,2,]

        class measure_report(utils_tests.edit_TestDefinition):
            description = 'pdf report for measure'
            url = base_url + '%s/pdf/' % self.measure.v_id
            users = [1,2,]
            data = {
                "submit_pdf": "Generate pdf",
            }

        class measure_edit(utils_tests.edit_TestDefinition):
            description = 'edit measure'
            url = base_url + '%s/edit/' % self.measure.v_id
            users = [1,]
            data = {
                "name": "Interlock",
                "description": "",
                "type": "Safety Instrumented System",
                "management-referer": base_url + '%s/description/' % self.measure.v_id
            }
            redirect_url = base_url + '%s/description/' % self.measure.v_id

        class measure_add(utils_tests.add_TestDefinition):
            description = 'add measure'
            url = '/analyses/%s/protectionlayers/%s/measure/add/' % (self.namespace, self.protectionlayer.v_id)
            users = [1,]
            data = {
                "name": "bogus",
                "description": "",
                "type": "some type",
                "management-referer": base_url
            }
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                pl = self.test_class.pl_model.objects.get(pk=self.test_class.protectionlayer.pk)
                self.check = pl.measures.count()
            def extra_tests(self, testcase, response, allowed):
                pl = self.test_class.pl_model.objects.get(pk=self.test_class.protectionlayer.pk)
                if allowed:
                    testcase.assertEquals(pl.measures.count(), self.check + 1)
                else:
                    testcase.assertEquals(pl.measures.count(), self.check)
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.plme_model.objects.order_by('-id')[0].delete(testcase.user, override=True)
                    self.test_class.me_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class measure_link(utils_tests.add_TestDefinition):
            description = 'link measure'
            url = '/analyses/%s/protectionlayers/%s/measure/add/' % (self.namespace, self.protectionlayer.v_id)
            users = [1,]
            data = {
                "link-measure": self.measure2.id,
                "management-referer": base_url,
                "link": 'link',
            }
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                pl = self.test_class.pl_model.objects.get(pk=self.test_class.protectionlayer.pk)
                self.check = pl.measures.count()
            def extra_tests(self, testcase, response, allowed):
                pl = self.test_class.pl_model.objects.get(pk=self.test_class.protectionlayer.pk)
                if allowed:
                    testcase.assertEquals(pl.measures.count(), self.check + 1)
                else:
                    testcase.assertEquals(pl.measures.count(), self.check)
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.plme_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class measure_delete(utils_tests.delete_TestDefinition):
            description = 'delete measure'
            users = [1,]
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                newitem = self.test_class.me_model(name='to_delete')
                newitem.save(defaultuser)
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                if not allowed:
                    self.test_class.me_model.objects.get(id=self.pk).delete(defaultuser, override=True)

        class measure_replace(utils_tests.edit_TestDefinition):
            description = 'replace measure'
            users = [1,]
            def pretest(self, testcase, allowed):
                newitem = self.test_class.me_model(name='to_replace')
                newitem.save(defaultuser)
                newitem2 = self.test_class.me_model(name='target')
                newitem2.save(defaultuser)
                newlink = self.test_class.plme_model(protectionlayer=self.test_class.protectionlayer, measure=newitem)
                newlink.save(defaultuser)
                self.pk = newitem.id
                self.v_id = newitem.v_id
                self.pk2 = newitem2.id
                self.v_id2 = newitem2.v_id
                self.url = base_url + '%s/replace/' % newitem.v_id
                self.data={'measure': newitem2.id}
                self.redirect_url = base_url + '%s/description/' % newitem2.v_id
            def extra_tests(self, testcase, response, allowed):
                pl = self.test_class.pl_model.objects.get(pk=self.test_class.protectionlayer.pk)
                if allowed:
                    testcase.assertEquals(self.test_class.plme_model.objects.filter(protectionlayer=pl, measure=self.v_id).count(), 0)
                    testcase.assertEquals(self.test_class.plme_model.objects.filter(protectionlayer=pl, measure=self.v_id2).count(), 1)
                else:
                    testcase.assertEquals(self.test_class.plme_model.objects.filter(protectionlayer=pl, measure=self.v_id).count(), 1)
                    testcase.assertEquals(self.test_class.plme_model.objects.filter(protectionlayer=pl, measure=self.v_id2).count(), 0)

            def posttest(self, testcase, allowed):
                self.test_class.plme_model.objects.order_by('-id')[0].delete(testcase.user, override=True)
                self.test_class.me_model.objects.get(id=self.pk2).delete(defaultuser, override=True)
                if not allowed:
                    self.test_class.me_model.objects.get(id=self.pk).delete(defaultuser, override=True)

        class measure_copy(utils_tests.edit_TestDefinition):
            description = 'copy measure'
            url = base_url + '%s/copy/' % self.measure.v_id
            users = [1,]
            def pretest(self, testcase, allowed):
                self.me_c = self.test_class.me_model.objects.count()
                self.ap_c = self.test_class.ap_model.objects.count()
                self.ind = self.test_class.measure.attentionpoints.count()
                self.redirect_url = base_url + '%s/description/' % self.test_class.me_model.objects.next_v_id()
            def posttest(self, testcase, allowed):
                if allowed:
                    testcase.assertEquals(self.test_class.me_model.objects.count(), self.me_c + 1)
                    testcase.assertEquals(self.test_class.ap_model.objects.count(), self.ap_c + self.ind)

        class measure_typelookup(utils_tests.TestDefinition):
            description = 'measure type lookup'
            url = base_url + 'type_lookup/'
            users = [1,2,]
            data = {
                "term": "strum",
            }
            contains = [
                {'url': 'Safety Instrumented System', 'users': [1,2,]},
            ]

        class measure_namelookup(utils_tests.TestDefinition):
            description = 'measure name lookup'
            url = base_url + 'name_lookup/'
            users = [1,2,]
            data = {
                "term": "nter",
            }
            contains = [
                {'url': 'Interlock', 'users': [1,2,]},
            ]

        tests = [
            measures(self),
            measures_report_select(self),
            measures_report(self),
            measures_csv(self),
            measure1_description(self),
            measure2_description(self),
            measure_attentionpoints(self),
            measure_usage(self),
            measure_compare(self),
            measure_copy(self),
            measure_report(self),
            measure_edit(self),
            measure_add(self),
            measure_link(self),
            measure_delete(self),
            measure_replace(self),
            measure_typelookup(self),
            measure_namelookup(self),
        ]

        utils_tests.run_tests(self, users, tests)

