"""
News app

"""
import datetime
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

class Entry(models.Model):
    user_created = models.ForeignKey(User, related_name="entries_created")
    user_changed = models.ForeignKey(User, related_name="entries_changed")
    date_created = models.DateTimeField()
    date_changed = models.DateTimeField()
    title = models.CharField(max_length=100)
    body = models.TextField(blank=True, null=True)

    class Meta:
        verbose_name = _('News entry')
        verbose_name_plural = _('News entries')
        ordering = ['-date_created']
        permissions = (
            ("display_news", "May display news entries"),
            ("edit_news", "May edit news entries"),
        )

    def __unicode__(self):
        return u'%s' % self.title

    # urls

    @models.permalink
    def get_absolute_url(self):
        return ('news_entry_display', (self.id,))

    @models.permalink
    def get_edit_url(self):
        return ('news_entry_edit', (self.id,))

    @models.permalink
    def get_delete_url(self):
        return ('news_entry_delete', (self.id,))


    # method overrides

    def save(self, user, *args, **kwargs):
        """
        Custom save method
        For totally new items:
            - set user and date for creation
        For other items:
            - set user and date for change
        """
        if not self.pk:
            self.user_created = user
            self.date_created = datetime.datetime.now()
        self.user_changed = user
        self.date_changed = datetime.datetime.now()
        super(Entry, self).save(*args, **kwargs)
    save.alters_data = True

    # permissions

    @classmethod
    def may_add(cls, user, parent=None):
        return user.has_perm('news.edit_news')

    def may_change(self, user):
        return user.has_perm('news.edit_news')

    def may_delete(self, user):
        return self.may_change(user)


from utils.search.models import searches, Search

class News_EntrySearch(Search):
    permission = 'news.display_news'
    fields = ['title', 'body']
    model = Entry

searches.register(News_EntrySearch)

